/* Keyboard Layer/2
 *
 * Select a keyboard layout
 * Copyright (C) 1999 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layout.cpp,v 1.3 2002/04/09 18:00:56 glass Exp $
 */

#include "getopt.h"
#include "layer_broker.h"
#include "layer_global.h"

#define  INCL_DOS
#define  INCL_ERRORS
#include <os2.h>
#include <stdlib.h>
#include <stdio.h>
#include <fstream.h>
#include <iomanip.h>

/*--------------------------------------------------
 * Errors
 *--------------------------------------------------*/
void error( ULONG error_code )
{
  char  msg[2048];
  ULONG len;

  DosGetMessage( 0, 0, msg, sizeof(msg), error_code, "OsO001.msg", &len );
  cerr << msg << endl;

  exit(1);
}

/*--------------------------------------------------
 * Usage
 *--------------------------------------------------*/
static void usage(void)
{
  fprintf( stderr, "Usage: layout <layout_name> [-c <codepage>] [-a]\n\n"

                   "  -c list available layouts for specified codepage\n"
                   "  -a list all available layouts\n"
         );

  exit(1);
}

/*--------------------------------------------------
 * Start here
 *--------------------------------------------------*/
void main( int argc, char **argv )
{
  cout << "Select a Keyboard Layout/2 Version "
       << PACKAGE_VERSION
       << endl
       << "Copyright (C) 1999-2002 by Dmitry A.Steklenev"
       << endl
       << endl;

  KLBroker broker;

  const KLBroker::cp_entry* cpList = broker.cp_list();
  int   c,
        cpage = cpList->selected;
  char  select_layout[MAX_LNAME] = "";

  while(( c = getopt( argc, argv, "c:ah?")) != -1 )
  {
    switch( c ) {
      case 'c': {
        if( !(cpage = atoi(optarg)))
          usage();
        break;
      }
      case 'a': {
        cpage = 0;
        break;
      }
      default:
        usage();
    }
  }

  while( optind < argc &&
         strlen(argv[optind]) < sizeof(select_layout) - strlen(select_layout) - 1)
  {
    strcat( select_layout, argv[optind++] );
  }

  if( optind < argc )
    usage();

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  // Display keyboard type

  USHORT keytype = broker.kb_type();

  cout << setw(15)
       << setiosflags( ios::left )
       << "Keyboard type"
       << " : "
       << keytype;

  switch( keytype )
  {
    case 0:
      cout << " (Personal computer AT keyboard)";
      break;

    case 1:
      cout << " (Enchanced keyboard)";
      break;

    default:
      cout << " (Unknown keyboard)";
  }

  cout << endl;

  // Display current and prepared codepages

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  cout << setw(15)
       << "Codepages"
       << " : "
       << "current "
       << cpList->selected;

  if( cpList->size )
  {
    cout << ", prepared";

    for( int i = 0; i < cpList->size; i++ )
      cout << " " << cpList->prepared[i];
  }

  cout << endl;

  // Display keyboard table file

  cout << setw(15)
       << "Keyboard file"
       << " : "
       << broker.table_name()
       << endl;

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  // Display keyboard layout

  cout << setw(15)
       << "Keyboard layout"
       << " : "
       << broker.selected()
       << endl;

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  // Display prepared keyboard layouts

  cout << endl
       << "List of prepared keyboard layouts ";

  if( cpage )
  {
    cout << "("
         << cpage
         << ")";
  }

  cout << ":"
       << endl
       << endl;

  KLBroker::kl_entry entry;
  broker.table_open();

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  while( broker.next( &entry, cpage ))
  {
    cout << "* "
         << setw(8)
         << entry.name
         << "  "
         << setw(4)
         << entry.cp;

    if( entry.dflag & DFLAG_DEFAULTFORCTRY )
      cout << " [default]";
    else
      cout << "          ";

    if( entry.dflag & DFLAG_MULTILAYERED )
      cout << " [multilayered]";

    cout << endl;
  }

  if( broker.error_code() != NO_ERROR )
    error( broker.error_code());

  broker.table_close();

  // Set keyboard layout

  if( strlen(select_layout))
  {
    cout << endl;
    broker.select( select_layout, TRUE );
  }
}

