/* Keyboard Layer/2
 *
 * Keyboard layouts broker class
 * Copyright (C) 1999 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_broker.h,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

/**@#-*/
#ifndef __LAYER_BROKER_H
#define __LAYER_BROKER_H

#define DFLAG_DEFAULTCPAGE    0x01 // Default codepage for country
#define DFLAG_MULTILAYERED    0x02 // Multilayered layout
#define DFLAG_SECONDARY       0x04 // Secondary multilayered layout
#define DFLAG_DEFAULTFORCTRY  0x08 // Default for country
#define MFLAG_MULTIPLE        0x01 // Multiple layout for country

/** Defines maximum size of the keyboard layout name. */
#define MAX_LNAME 10

#include <fstream.h>
#include <os2def.h>
/**@#+*/

/**
 * Keyboard layouts broker class.
 * @author Dmitry A Steklenev
 */

class KLBroker
{
  public:
    /** Constructs objects of the broker class. */
    KLBroker();
    /** Destructs objects of the broker class. */
   ~KLBroker();

    #pragma pack(1)

    /** Index entry of the keyboard.dcp file. */
    struct kl_entry {

      char   name[8];       //@- Includes subcountry id.
      SHORT  cp;            //@- Code page number.
      USHORT kbtype;        //@- Keyboard type.
      ULONG  trans;         //@- Offset of translate table in the file.
      UCHAR  dflag;         //@- Default translate table for this cp.
      UCHAR  mflag;         //@- Machine dependent keyboard.
    };

    /** Codepages entry. */
    struct cp_entry {

      ULONG  size;          //@- Count of the prepared codepages.
      ULONG  selected;      //@- Current selected codepage.
      ULONG  prepared[10];  //@- Prepared codepages.
    };

    #pragma pack()

    /** Use these enumerators to specify keyboard layout's type. */
    enum type { any, primary, secondary };

  public:
    /** Returns the error code set by the failure of a broker method. */
    ULONG error_code() const;

    /**
     * Returns a type of attached keyboard.
     *
     * @return 0x0000 - personal computer AT keyboard,
     *         0x0001 - enchanced keyboard,
     *         0xFFFF - error
     */

    USHORT kb_type();

    /** Returns a list of the prepared codepages. */
    const cp_entry* cp_list();
    /** Returns a name of the selected keyboard layout. */
    const char* selected();

    /** Returns a name of the keyboard layouts file. */
    const char* table_name();
    /** Opens  a keyboard layouts file. */
    ULONG table_open ();
    /** Closes a keyboard layouts file. */
    ULONG table_close();

    /** Returns the next keyboard layout. */
    BOOL next( kl_entry* entry, ULONG cp = 0, type tp = primary );
    /** Returns the specified keyboard layout. */
    BOOL find( kl_entry* entry, const char* name, ULONG cp = 0, type tp = primary );
    /** Selects specifed keyboard layout. */
    BOOL select( const char* name, BOOL verbose = FALSE );

    /** Is specified layout multilayered. */
    BOOL is_multilayered( const char* name, ULONG cp );
    /** Is current selected layout multilayered. */
    BOOL is_multilayered();

  private:
    /** Removes leading and trailing spaces. */
    char* strip( char *s );
    /** Opens a keyboard device driver.  */
    ULONG kb_open ();
    /** Closes a keyboard device driver. */
    ULONG kb_close();
    /** Sets translation table for VIO, FS and VDM. */
    ULONG select_fs( const kl_entry& entry, ULONG index, BOOL verbose );
    /** Sets translation table for PM. */
    BOOL  select_pm( const kl_entry& entry, BOOL verbose );

  private:
    struct info
    {
      char* name;
      ULONG pri_id;
      ULONG sec_id;
    };

    static const info lb_list[];

    ULONG    lb_error;
    cp_entry lb_cp;
    HFILE    lb_handle;
    char*    lb_table;
    char     lb_name[10];

    unsigned tb_open;     //@- > 0, if file is open.
    ifstream tb_file;
    USHORT   tb_size;     //@- Count of a keyboard layouts in layouts file.
    ULONG    tb_entry;    //@- Layouts file entry point.

    ULONG    search_pos;  //@- Current keyboard layout number in read() operation.
    ULONG    search_ofs;  //@- Current file offset for read() operation.
};

#if !defined(__ccdoc__)
/*--------------------------------------------------
 * Returns the error code set by the failure
 *--------------------------------------------------*/
inline ULONG KLBroker::error_code() const
{
  return lb_error;
}
#endif

/**@#-*/
#endif
/**@#+*/
